﻿const string[] g_sfxNames =
{
	/*	0	*/	"soundfx/button.mp3",
	/*	1	*/	"soundfx/explosion_small.mp3",
	/*	2	*/	"soundfx/portal_fail.mp3",
	/*	3	*/	"soundfx/portal_reflect.mp3",
	/*	4	*/	"soundfx/demolition.mp3",
	/*	5	*/	"soundfx/fireball.mp3",
	/*	6	*/	"soundfx/portal_killed.mp3",
	/*	7	*/	"soundfx/teleport.mp3",
	/*	8	*/	"soundfx/explosion_huge.mp3",
	/*	9	*/	"soundfx/portal_created.mp3",
	/*	10	*/	"soundfx/portal_launch.mp3",
	/*	11	*/	"soundfx/final_door_sound.mp3",
	/*	12	*/	"soundfx/stone_hit.mp3",
	/*	13	*/	"soundfx/elevator.mp3",
	/*	14	*/	"soundfx/burn.mp3",
	/*	15	*/	"soundfx/wood01.mp3",
	/*	16	*/	"soundfx/wood02.mp3",
	/*	17	*/	"soundfx/door_open.mp3",
	/*	18	*/	"soundfx/fall.mp3",
	/*	19	*/	"soundfx/projectile_reflect.mp3",
	/*	20	*/	"soundfx/magical_sweep_08.mp3",
	/*	21	*/	"soundfx/dark_whoosh_17.mp3",
	/*	22	*/	"soundfx/wood_drag.mp3",
	/*	23	*/  "soundfx/crystal_gather_G.mp3",
	/*	24	*/  "soundfx/crystal_gather_E.mp3",
	/*	25	*/  "soundfx/spike_hit.mp3",
	/*	26	*/  "soundfx/crystal_temp_alert.mp3",
	/*	27	*/  "soundfx/enemy_spotted.mp3",
	/*  28  */  "soundfx/flesh_heavy_impact_03.mp3",
	/*  29  */  "soundfx/orchestral_transition_stinger_06.mp3"
};

const string[] g_musicNames =
{
	/*	0	*/	"soundfx/running_with_wolves_loop_b.mp3",
	/*	1	*/	"soundfx/finding_wonderland_60_loop.mp3",
	/*	2	*/	"soundfx/warlords_loop_a.mp3",
	/*	3	*/  "soundfx/rocket_space_shuttle_rocket_distant.mp3",
	/*	4	*/	"soundfx/back_to_nestopia.mp3"
};

const uint __BOSS_MUSIC_NAME = 0;
const uint __GAME_MUSIC_NAME = 1;
const uint __MENU_MUSIC_NAME = 2;
const uint __ROCKET_MUSIC_NAME = 3;
const uint __BACK_HOME_MUSIC_NAME = 4;

const float GAME_MUSIC_VOLUME = 0.7f;
const float BOSS_MUSIC_VOLUME = 0.6f;

class AudioManager : GameController, GlobalVolumeSwitchListener, MusicSwitchListener
{
	private Timer charHitSoundTimer;
	private Timer fireballSoundTimer;
	private Timer smallExplosionSoundTimer;
	private Timer hugeExplosionSoundTimer;
	private Timer woodSoundTimer;
	private Timer crystalSoundTimer;
	private Timer woodDragSoundTimer;
	private string currentTrack;

	AudioManager()
	{
	}

	void update()
	{
		g_globalVolumeManager.setSwitchListener(this);
		g_musicSwitchManager.setSwitchListener(this);
		fireballSoundTimer.update();
		smallExplosionSoundTimer.update();
		hugeExplosionSoundTimer.update();
		woodSoundTimer.update();
		woodDragSoundTimer.update();
		charHitSoundTimer.update();
		crystalSoundTimer.update();
	}

	void onResume()
	{
		loadSounds();
		if (g_musicSwitchManager.getState())
			resumeMusic();
	}

	void turnMusicOff()
	{
		pauseMusic();
	}

	void turnMusicOn()
	{
		resumeMusic();
	}

	void turnGlobalVolumeOff()
	{
		pauseMusic();
	}

	void turnGlobalVolumeOn()
	{
		if (g_musicSwitchManager.getState())
			resumeMusic();
	}

	private void pauseMusic()
	{
		if(currentTrack != "")
		{
			if (SampleExists(currentTrack))
			{
				LoopSample(currentTrack, true); // don't remember why I did that
				StopSample(currentTrack);
			}
		}
	}

	private void resumeMusic()
	{
		LoadMusic(currentTrack);
		LoopSample(currentTrack, true);
		PlaySample(currentTrack);
	}

	void draw() {}

	void loadSounds()
	{
		for (uint t = 0; t < g_sfxNames.length(); t++)
		{
			LoadSoundEffect(g_sfxNames[t]);
		}
	}

	string getRetryButtonSound()
	{
		return (g_sfxNames[7]);
	}

	void playVictorySound()
	{
		SetSampleSpeed(g_sfxNames[20], 1.0f);
		PlaySample(g_sfxNames[20]);
	}
	
	void playRoundaboutSound()
	{
		SetSampleSpeed(g_sfxNames[20], 0.6f);
		PlaySample(g_sfxNames[20]);
		SetSampleSpeed(g_sfxNames[9], 0.6f);
		PlaySample(g_sfxNames[9]);
	}

	void playLightOnSound()
	{
		SetSampleSpeed(g_sfxNames[20], 0.6f);
		PlaySample(g_sfxNames[20]);
		SetSampleSpeed(g_sfxNames[9], 0.6f);
		PlaySample(g_sfxNames[9]);
	}

	void playLightOffSound()
	{
		SetSampleSpeed("soundfx/reverse_magical_sweep_08.mp3", 0.6f);
		PlaySample("soundfx/reverse_magical_sweep_08.mp3");
		SetSampleSpeed(g_sfxNames[9], 0.6f);
		PlaySample(g_sfxNames[9]);
	}

	void playDoorUnlockSound() const
	{
		SetSampleSpeed(g_sfxNames[23], 0.5f);
		PlaySample(g_sfxNames[23]);
	}

	void playEnemySpottedSound()
	{
		PlaySample(g_sfxNames[27]);
	}

	void playDeathSound()
	{
		PlaySample(g_sfxNames[21]);
	}
	
	void playAchievementSound()
	{
		PlaySample(g_sfxNames[29]);
	}
	
	string getAchievementSoundName()
	{
		return g_sfxNames[29];
	}
	
	void playCrystalPickSound()
	{
		if (crystalSoundTimer.getTime() > 50)
		{
			uint idx = 23 + rand(1);
			SetSampleVolume(g_sfxNames[idx], 0.8f);
			SetSampleSpeed(g_sfxNames[idx], 1.0f);
			PlaySample(g_sfxNames[idx]);
			crystalSoundTimer.reset();
		}
	}

	void playKeyPickSound()
	{
		const uint idx = 24;
		SetSampleVolume(g_sfxNames[idx], 1.0f);
		SetSampleSpeed(g_sfxNames[idx], 0.5f);
		PlaySample(g_sfxNames[idx]);
	}

	void playKeyUnlockSound()
	{
		const uint idx = 25;
		SetSampleVolume(g_sfxNames[idx], 1.0f);
		SetSampleSpeed(g_sfxNames[idx], 0.5f);
		PlaySample(g_sfxNames[idx]);
	}
	
	void playLightSwitchSound()
	{
		playKeyUnlockSound();
	}

	void playCrystalTempAlertSound()
	{
		if (crystalSoundTimer.getTime() > 100)
		{
			PlaySample(g_sfxNames[26]);
			crystalSoundTimer.reset();
		}
	}

	void playCrystalVanishSound()
	{
		SetSampleSpeed(g_sfxNames[25], 1.0f);
		PlaySample(g_sfxNames[25]);
	}

	void playReflectSound()
	{
		PlaySample(g_sfxNames[19]);
	}

	void playEnemyShootSound()
	{
		PlaySample(g_sfxNames[19]);
	}

	void playFallSound(const float volume)
	{
		if (charHitSoundTimer.getTime() > 200)
		{
			SetSampleVolume(g_sfxNames[18], volume);
			PlaySample(g_sfxNames[18]);
			charHitSoundTimer.reset();
		}
	}

	void playDieByFallSound()
	{
		SetSampleVolume(g_sfxNames[18], 1.0f);
		PlaySample(g_sfxNames[18]);
	}

	void playMinionFallSound()
	{
		PlaySample(g_sfxNames[28]);
	}

	void playMinionSmashSound()
	{
		PlaySample(g_sfxNames[28]);
	}

	void playRandomWoodSound(const float volume)
	{
		if (woodSoundTimer.getTime() > 150)
		{
			const string name = (g_sfxNames[15 + rand(1)]);
			SetSampleVolume(name, volume);
			SetSampleSpeed(name, randF(0.2f) + 0.8f);
			PlaySample(name);
			woodSoundTimer.reset();
		}
	}

	void playBurnSound()
	{
		PlaySample(g_sfxNames[14]);
	}

	void playFireballSound()
	{
		if (fireballSoundTimer.getTime() > 100)
		{
			PlaySample(g_sfxNames[5]);
			fireballSoundTimer.reset();
		}
	}

	string getRestartLevelButtonSound()
	{
		return g_sfxNames[7];
	}

	string getExitLevelButtonSound()
	{
		return g_sfxNames[7];
	}

	string getSkipLevelButtonSound()
	{
		return g_sfxNames[7];
	}

	string getNextLevelButtonSound()
	{
		return g_sfxNames[7];
	}

	string getItemSelectButtonSoundName()
	{
		return g_sfxNames[7];
	}

	void playElevatorSound()
	{
		PlaySample(g_sfxNames[13]);
	}

	void playDoorOpenSound(const uint doorOpenStride)
	{
		SetSampleSpeed(g_sfxNames[17], 3000.0f / float(doorOpenStride));
		PlaySample(g_sfxNames[17]);
	}

	void playDoorCloseSound(const uint doorOpenStride)
	{
		SetSampleSpeed(g_sfxNames[17], 3000.0f / float(doorOpenStride));
		PlaySample(g_sfxNames[17]);
	}

	void playExplosionSound(const bool huge)
	{
		if (huge)
		{
			if (hugeExplosionSoundTimer.getTime() > 150)
			{
				PlaySample(g_sfxNames[8]);
				hugeExplosionSoundTimer.reset();
			}
		}
		else
		{
			if (smallExplosionSoundTimer.getTime() > 150)
			{
				PlaySample(g_sfxNames[1]);
				smallExplosionSoundTimer.reset();
			}
		}
	}

	void playPortalCreatedSound() const
	{
		SetSampleSpeed(g_sfxNames[9], 1.0f);
		PlaySample(g_sfxNames[9]);
	}

	void playPortalLaunchSound() const
	{
		PlaySample(g_sfxNames[10]);
	}
	
	void playPortalKilledSound() const
	{
		PlaySample(g_sfxNames[6]);
	}

	void playPortalFailedSound() const
	{
		PlaySample(g_sfxNames[2]);
	}

	void playEnemyShootExplodeSound() const
	{
		PlaySample(g_sfxNames[2]);
	}

	void playTeleportSound() const
	{
		PlaySample(g_sfxNames[7]);
	}

	void playStoneAppearSound() const
	{
		SetSampleSpeed(g_sfxNames[9], 1.0f);
		PlaySample(g_sfxNames[9]);
	}
	
	void playWoodDragSound()
	{
		if (woodDragSoundTimer.getTime() > 200)
		{
			SetSampleSpeed(g_sfxNames[2], 0.5f + randF(1.5f));
			PlaySample(g_sfxNames[22]);
			woodDragSoundTimer.reset();
		}
	}
	
	void playFinalDoorSound() const
	{
		PlaySample(g_sfxNames[11]);
	}
	
	void playLevelFinishedSound() const
	{
		PlaySample(g_sfxNames[7]);
	}

	void playStoneHitSound(const float volume)
	{
		SetSampleVolume(g_sfxNames[12], volume);
		SetSampleSpeed(g_sfxNames[12], randF(0.2f) + 0.8f);
		PlaySample(g_sfxNames[12]);
	}

	void playDemolitionSound()
	{
		PlaySample(g_sfxNames[4]);
	}

	void playBeholderHitSound()
	{
		PlaySample(g_sfxNames[4]);
	}

	string getButtonSoundName() const
	{
		return g_sfxNames[0];
	}
	
	void playAchievementPickSound()
	{
		PlaySample(g_sfxNames[0]);
	}

	void playAchievementDismissButtonSound()
	{
		PlaySample(g_sfxNames[6]);
	}

	void playAchievementPickDeniedSound()
	{
		PlaySample(g_sfxNames[6]);
	}

	string getStartAchievementSoundName()
	{
		return (g_sfxNames[7]);
	}

	string getMenuMusicName()
	{
		return g_musicNames[__MENU_MUSIC_NAME];
	}

	string getRocketMusicName()
	{
		return g_musicNames[__ROCKET_MUSIC_NAME];
	}

	string getGameMusicName()
	{
		return g_musicNames[__GAME_MUSIC_NAME];
	}
	
	string getBossMusicName()
	{
		return g_musicNames[__BOSS_MUSIC_NAME];
	}

	void setMusicVolume(const float volume)
	{
		if(currentTrack != "")
		{
			if (SampleExists(currentTrack))
			{
				SetSampleVolume(currentTrack, volume);
			}
		}
	}

	void setDefaultMusicVolume()
	{
		if(currentTrack != "")
		{
			if (SampleExists(currentTrack))
			{
				SetSampleVolume(currentTrack, (currentTrack == g_musicNames[__BOSS_MUSIC_NAME]) ? BOSS_MUSIC_VOLUME : GAME_MUSIC_VOLUME);
			}
		}
	}

	void playMusic(const string &in name, const float volume, const bool loop = true)
	{
		currentTrack = name;
		#if DESKTOP
		for (uint t = 0; t < g_musicNames.length(); t++)
		{
			if (SampleExists(g_musicNames[t]))
				StopSample(g_musicNames[t]);
		}
		#endif
		if (!g_musicSwitchManager.getState())
			return;
		LoadMusic(name);
		LoopSample(name, loop);
		SetSampleVolume(name, volume);
		PlaySample(name);
	}

	void playMenuMusic()
	{
		playMusic(g_musicNames[__MENU_MUSIC_NAME], 1.0f);
	}

	void playRocketMusic()
	{
		playMusic(g_musicNames[__ROCKET_MUSIC_NAME], 1.0f, false);
	}

	void playBackHomeMusic()
	{
		playMusic(g_musicNames[__BACK_HOME_MUSIC_NAME], 1.0f, false);
	}

	void playGameMusic(const bool bossFight)
	{
		if (bossFight)
			playMusic(g_musicNames[__BOSS_MUSIC_NAME], BOSS_MUSIC_VOLUME);
		else
			playMusic(g_musicNames[__GAME_MUSIC_NAME], GAME_MUSIC_VOLUME);
	}

	void playGhostItemAppearSound() const
	{
		SetSampleSpeed(g_sfxNames[9], 1.0f);
		PlaySample(g_sfxNames[9]);
	}

	void playBeholderAppearSound()
	{
		PlaySample("soundfx/beholder_appear.mp3");
	}

	void playGhostAppearSound()
	{
		PlaySample("soundfx/beholder_appear.mp3");
		PlaySample("soundfx/ghost_appear_whoosh.mp3");
	}

	void playBeholderDamageSound()
	{
		SetSampleSpeed("soundfx/beholder_damage.mp3", 1.0f);
		PlaySample("soundfx/beholder_damage.mp3");
	}

	void playGhostDamageSound()
	{
		PlaySample("soundfx/beholder_appear.mp3");
	}

	void playGhostDeathSound()
	{
		SetSampleSpeed("soundfx/beholder_damage.mp3", 0.5f);
		PlaySample("soundfx/beholder_damage.mp3");
	}

	void playDragonTakeOffSound()
	{
		PlaySample("soundfx/dragon_lost.mp3");
	}

	void playDarkDragonDeathSound()
	{
		PlaySample("soundfx/dragon_lost.mp3");
	}

	void playDarkDragonOpenMouthSound()
	{
		PlaySample("soundfx/dragon_lost.mp3");
	}

	void playDarkDragonAppearSound()
	{
		PlaySample("soundfx/beholder_damage.mp3");
	}

	void playDarkDragonDamageSound()
	{
		PlaySample("soundfx/beholder_damage.mp3");
	}

	void playDragonAppearSound()
	{
		SetSampleSpeed("soundfx/beholder_damage.mp3", 1.0f);
		PlaySample("soundfx/beholder_damage.mp3");
	}

	void playBeholderSpikesSound()
	{
		SetSampleSpeed("soundfx/spikes.mp3", 1.0f);
		PlaySample("soundfx/spikes.mp3");
	}

	void playFireExtinguishSound()
	{
		SetSampleSpeed("soundfx/spikes.mp3", 0.5f);
		PlaySample("soundfx/spikes.mp3");
	}

	void playBeholderSpikeHitSound()
	{
		SetSampleSpeed(g_sfxNames[25], 1.0f);
		PlaySample(g_sfxNames[25]);
	}

	void playBeholderCloseEyesSound()
	{
		PlaySample("soundfx/beholder_appear.mp3");
	}

	void playShockDiamondPickSound()
	{
		PlaySample("soundfx/shock_diamond_pick.mp3");
	}

	void playShockStrikeSound()
	{
		PlaySample("soundfx/shock_strike.mp3");
	}
	
	void playMinionQuestionMarkSound()
	{
		PlaySample("soundfx/enemy_confused.mp3");
	}

	void loadSpecificSounds(const uint world, const uint level)
	{
		if (world == 0 && level == 31)
		{
			LoadSoundEffect("soundfx/beholder_appear.mp3");
			LoadSoundEffect("soundfx/beholder_damage.mp3");
			LoadSoundEffect("soundfx/spikes.mp3");
		}
		if (world == 1 && level == 31)
		{
			LoadSoundEffect("soundfx/beholder_damage.mp3");
			LoadSoundEffect("soundfx/dragon_lost.mp3");
		}
		if (world == 2)
		{
			LoadSoundEffect("soundfx/enemy_confused.mp3");
			if ((level == 15 || level == 16 || level == 17 || level == 19 || level == 23
				|| level == 24 || level == 25 || level == 26 || level == 27 || level == 30 || level == 31))
			{
				LoadSoundEffect("soundfx/shock_diamond_pick.mp3");
				LoadSoundEffect("soundfx/shock_strike.mp3");
			} 
			if ((level == 30 || level == 29 || level == 31))
			{
				LoadSoundEffect("soundfx/spikes.mp3");
				if (level == 31)
				{
					LoadSoundEffect("soundfx/beholder_damage.mp3");
					LoadSoundEffect("soundfx/beholder_appear.mp3");
					LoadSoundEffect("soundfx/ghost_appear_whoosh.mp3");
				}
			}
		}
		if (world == 3)
		{
			LoadSoundEffect("soundfx/shock_diamond_pick.mp3");
			LoadSoundEffect("soundfx/shock_strike.mp3");

			if (level == 18)
			{
				LoadSoundEffect("soundfx/red_key_got.mp3");
			}
			else if (level >= 24)
			{
				LoadSoundEffect("soundfx/enemy_confused.mp3");
			}
			if (level >= 24)
			{
				LoadSoundEffect("soundfx/spikes.mp3");
				LoadSoundEffect("soundfx/reverse_magical_sweep_08.mp3");
			}
			if (level == 31)
			{
				LoadSoundEffect("soundfx/beholder_damage.mp3");
				LoadSoundEffect("soundfx/dragon_lost.mp3");
			}
		}
	}

	void playRedKeyGotSound()
	{
		PlaySample("soundfx/red_key_got.mp3");
	}

	void unloadSpecificSounds(const uint world, const uint level)
	{
		if (world == 0 && level == 31)
		{
			// TO-DO
		}
	}
}

AudioManager g_audioManager;
